#!/usr/bin/python
#
# test_filelist.py
# A unit test suite for filelist.py
#
# (c) Simeon Veldstra 2006  <reallifesim@gmail.com>
#
# This software is free.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You may redistribute this program under the terms of the
# GNU General Public Licence version 2
# Available in this package or at http://www.fsf.org


"""
FileList must:
addfiles() must accept a single filename or a sequence
 containing filenames. The files must exist and must be valid
 input files.
addplaylist() must accept a filename and read the file to load
 any files listed within.
addXMMS([instance]) should, if possible, load the playlist from 
 the running instance of XMMS.
removefile(file) must remove the named file from the list, if 
 it exists.
__iter__() a generator function to iterate through the files in
 the list.
cur_file() must return the current iterator position in the list.
__len__() must return the length of the list.
__contains__(file) must return True iff the file is in the list.
"""

banner = """  test_filelist:

The XMMS playlist test requires that XMMS be running and that
session 0 (The first instance started) has the test file set loaded 
as its playlist. You can accomplish this by importing the top level 
test data directory into XMMS's playlist editor with no other 
copies of XMMS running. 

If XMMS is not running, the test suite will print a warning and
continue. If XMMS is running with a different playlist, the test 
will fail and the rest of the tests will continue.

""" 

import sys, os
import unittest

# Get the modules from the package we are in, rather
# than any others of the same name installed to the
# system so we can test before installing.
if not sys.path[0] == "../":
	sys.path.insert(0, "../")

import mp3togo.filelist as filelist
import mp3togo.conf as setup

### Set up test data ###

import test_metadata
data = test_metadata.metadata()

### Start of Tests ###

class FileListAdd(unittest.TestCase):
	"""Add files to the list
	Test ogg, mp3, flac, wav"""

	def single(self, name):
		lst = filelist.FileList()
		lst.addfiles(data[name]['file'])
		self.assert_(setup.checkfile(data[name]['file']) in lst._list)
		self.assert_(len(lst) == 1)

	def testAddOgg(self):
		"""FileList: add ogg vorbis file"""
		self.single('ogg')

	def testAddMp3(self):
		"""FileList: add mp3 file"""
		self.single('mp3')

	def testAddFlac(self):
		"""FileList: add flac file"""
		self.single('flac')
	
	def testAddWav(self):
		"""FileList: add wav file"""
		self.single('wav')
	
	def testAddMultiple(self):
		"""FileList: add multiple files"""
		fl = []
		testfiles = ('ogg', 'mp3', 'flac', 'wav', 'bareogg', 'baremp3')
		for name in testfiles:
			fl.append(setup.checkfile(data[name]['file']))
		lst = filelist.FileList()
		lst.addfiles(fl)
		if False in map(lambda x: x in lst, fl):
			raise AssertionError
		if len(testfiles) != len(lst):
			raise AssertionError


class FileListAddPlaylist(unittest.TestCase):
	"""Add playlists
	Test m3u and pls"""

	def setUp(self):
		test_metadata.absolutize_playlists(data)

	def tearDown(self):
		test_metadata.remove_local_playlists(data)

	def playlist(self, name):
		lst = filelist.FileList()
		lst.addplaylist(data[name]['file'])
		self.assertEqual(data[name]['content'], lst._list)
		
	def testAddM3u(self):
		"""FileList: add m3u playlist"""
		self.playlist('absm3u')

	def testAddPls(self):
		"""FileList: add pls playlist"""
		self.playlist('abspls')

	def testXMMS(self):
		"""FileList: add playlist from XMMS
		Load the test_data directory into XMMS first"""
		try:
			import xmms.control
		except ImportError:
			print "xmms.control not available - skipping"
			return
		if not xmms.control.is_running():
			print "XMMS Not Running"
			return
		lst = filelist.FileList()
		lst.addXMMS()
		reflst = filelist.FileList()
		reflst.addplaylist(data['absm3u']['file'])
		lst._list.sort()
		reflst._list.sort()
		self.assertEqual(lst._list, reflst._list)


class FileListRemoveFile(unittest.TestCase):
	"""Remove files"""

	def testRemoveFile(self):
		"""FileList: remove file from list"""
		lst = filelist.FileList()
		fname = setup.checkfile(data['mp3']['file'])
		lst.addfiles(fname)
		self.assert_(fname in lst._list)
		lst.removefile(fname)
		self.assert_(fname not in lst._list)


class FileListListMethods(unittest.TestCase):
	"""Behave like a list"""

	def setUp(self):
		test_metadata.absolutize_playlists(data)
		self.lst = filelist.FileList()
		self.lst.addplaylist(data['absm3u']['file'])

	def tearDown(self):
		test_metadata.remove_local_playlists(data)

	def testIter(self):
		"""FileList: iterate through list"""
		outp = []
		self.lst.run()
		for f in self.lst:
			outp.append(f)
		self.assertEqual(data['absm3u']['content'], outp)

	def testCur_file(self):
		"""FileList: check current file pointer"""
		self.lst.run()
		it = self.lst.__iter__()
		for i in range(len(data['absm3u']['content'])):
			self.assertEqual(data['absm3u']['content'][i], it.next())
			self.assertEqual(self.lst.cur_file(), i)

	def testLen(self):
		"""FileList: check len()"""
		self.assertEqual(len(data['absm3u']['content']), len(self.lst))

	def testContains(self):
		"""FileList: check __contains__"""
		for i in data['absm3u']['content']:
			self.assert_(i in self.lst)
		
	
### End of Tests ###

def suite():
	suite = unittest.TestSuite()
	suite.addTest(unittest.makeSuite(FileListAdd))
	suite.addTest(unittest.makeSuite(FileListAddPlaylist))
	suite.addTest(unittest.makeSuite(FileListRemoveFile))
	suite.addTest(unittest.makeSuite(FileListListMethods))
	return suite

if __name__ == "__main__":
	unittest.main()
