#!/usr/bin/python
#
# test_metadata.py
# A unit test suite for options.py
#
# (c) Simeon Veldstra 2006  <reallifesim@gmail.com>
#
# This software is free.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You may redistribute this program under the terms of the
# GNU General Public Licence version 2
# Available in this package or at http://www.fsf.org


"""
Metadata for the test dataset

Holds the expected values for the unit tests. These
values are dependant on the test files used, so they
are separated out into this module. This data must be
kept in sync with the test files in test_dir if you 
want the unit tests to work.
"""

import sys, os
import shutil

# Get the modules from the package we are in, rather
# than any others of the same name installed to the
# system so we can test before installing.
if not sys.path[0] == "../":	
	sys.path.insert(0, "../")

import mp3togo.options as options


### Set up test data ###

def metadata():
	testdir = "../../test_data/"
	data = {
	  'mp3':  {
	    'file': 'test_artist/test_album/testmp3.mp3', 
	    'ARTIST': u'DJ Fish Finger', 
		'ALBUM': u"Gone Fishin'", 
		'TITLE': u'Fishing Without Conviction', 
		'TRACKNUMBER': '',
		'GENRE': 'Electronic',
		'GENRE_NAME': 'Electronic', 
		'file_outname': 'player/test_album/testmp3-TEST-SUITE-.mp3',
		'file_outdir': 'player/test_album',
		'file_wavname': 'temp/testmp3-TEST-SUITE-.wav'},
	  'ogg':  {
	    'file': 'test_artist/test_album/testogg.ogg', 
	    'ARTIST': u'DJ Fish Finger', 
		'ALBUM': u"Gone Fishin'", 
		'TITLE': u'Fishing Without Conviction', 
		'TRACKNUMBER': ''}, 
	  'flac': {
	    'file': 'test_artist/test_album/testflac.flac',  
	    'ARTIST': u'DJ Fish Finger', 
		'ALBUM': u"Gone Fishin'", 
		'TITLE': u'Fishing Without Conviction', 
		'TRACKNUMBER': u'4'},
	  'wav':  {
	    'file': 'test_artist/test_album/testwav.wav',  
	    'ARTIST': u'test artist', 
		'ALBUM': u'test album', 
		'TITLE': u'testwav', 
		'TRACKNUMBER': ''},
	  'notmp3':  {
	    'file': 'testunknown.txt'},
	  'baremp3':  {
	    'file': 'testbare.mp3'},
	  'bareogg':  {
	    'file': 'testbare.ogg'},
	  'm3u':  {
	    'file': 'test_playlist.m3u',
	    'len': 6},
	  'pls':  {
	    'file': 'test_playlist.pls'},
	  'gone':  {
	    'file': 'this_file_does_not_exist.gone'}}

	for key in data.keys():
		for fkey in filter(lambda x: x.startswith('file'), data[key].keys()):
			data[key][fkey] = os.path.abspath(os.path.join(testdir, data[key][fkey]))
	
	return data

class t_opts(options.Options):
	"""Options is used by subclassing and extending."""

	def __init__(self, argv=None, conffile=None, readconf=True):
		options.Options.__init__(self, test_conffile)
		# Add some options to test:
		self._set += [
		 ('test1', 'x', 'test-1', 'default-1', self._hook1, "Test 1"),
		 ('test2', 'y', 'test-2', '', None, "Test 2"),
		 ('test3', 'z', 'test-3', 0, None, "Test 3"),
		 ('test4', '', 'test-4', 0.0, None, "Test 4"),
		 ('test5', '', '', False, None, "Test 5"),
		 ('test_nosave', 'w', 'test-no_save', 0, None, "Test No Save")]

		# Don't save these options:
		self._nosave_options += ['test_nosave']

		# Check for these binaries:
		self.bin['python'] = None  # Should be on any system this runs on.
		self.bin['This_Binary_Does_Not_Exist'] = None # Hopefully dosen't exist

		# Help text:
		self._help_preamble = "This is the test suite for\nmp3togo.options.\n\n"

		self.getconf(argv, conffile, readconf)

	def _hook1(self, value):
		# test1 must have a string value!
		self._d['test1'] = value.replace('ADDED_BY_HOOK', '') + "ADDED_BY_HOOK"

	def _pre_hook(self):
		self._pre_hook_ran = True

	def _post_hook(self):
		self._post_hook_ran = True

test_conffile = './test_options.cfg'
test_alt_conffile = './test_options.alt.cfg'

def clean_config():
	"""Clean up the test config file if it exists."""
	if os.path.exists(test_conffile):
		os.unlink(test_conffile)
	if os.path.exists(test_alt_conffile):
		os.unlink(test_alt_conffile)



def absolutize_playlists(data):
	def tname(name):
		return os.path.join(os.path.split(name)[0], "abs" + os.path.split(name)[1])
	def createfile(kind):
		absn = tname(data[kind]['file'])
		shutil.copyfile(data[kind]['file'], absn)
		inp = file(data[kind]['file'], 'r')
		op = file(absn, 'w')
		content = []
		for line in inp:
			if line.startswith('../..'):
				nl = os.path.abspath(line.strip()) 
				content.append(nl)
				nl = nl + '\n'
			elif line.startswith("File") and line.find('=') > -1:
				pre, name = line.strip().split('=', 1)
				name = os.path.abspath(name)
				content.append(name)
				nl = pre + '=' + name + '\n'
			else:
				nl = line
			op.write(nl)
		inp.close()
		op.close()
		data['abs' + kind] = {'file': absn, 'content': content}
	for kind in ('m3u', 'pls'):
		createfile(kind)

def remove_local_playlists(data):
	for kind in ('absm3u', 'abspls'):
		os.unlink(data[kind]['file'])
		del data[kind]

def indexfile():
	"""return the expected contents of index.out

	Assumes use of data['mp3']['file'] to produce the index file."""

	data = metadata()
	return data['mp3']['file'] + \
	       "\nALBUM: %s\nARTIST: %s\nGENRE: %s\nTITLE: %s\n\n" % \
		   (data['mp3']['ALBUM'], data['mp3']['ARTIST'], 
		    data['mp3']['GENRE'], data['mp3']['TITLE'])




