#!/usr/bin/python
#
# test_options.py
# A unit test suite for options.py
#
# (c) Simeon Veldstra 2006  <reallifesim@gmail.com>
#
# This software is free.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You may redistribute this program under the terms of the
# GNU General Public Licence version 2
# Available in this package or at http://www.fsf.org


"""
Options must:
  Set settings from command line options.
    Fail on unknown options.
  Save to and restore from a specified config file.
  	Read from correct file.
	Write to correct file.
	Ignore unknown entries.
	Don't save items in the _nosave_options list.
	Respect the readconf and conffile arguments.
  Accept and save only valid configuration settings.
    Check type.
  Run hook functions if they exist.
    On setting mutation.
	On pre and post configuration.
  Return configuration settings.
  Check for required binaries.
  Produce a help message.
    Show only options with flags.
  Manage log messages.
    Verbosity levels and quiet.

"""

import sys, os
import unittest

import test_metadata

# Get the modules from the package we are in, rather
# than any others of the same name installed to the
# system so we can test before installing.
if not sys.path[0] == "../":
	sys.path.insert(0, "../")

import mp3togo.options as options

test_conffile = test_metadata.test_conffile
test_alt_conffile = test_metadata.test_alt_conffile
t_opts = test_metadata.t_opts

clean_config = test_metadata.clean_config
clean_alt_config = test_metadata.clean_config

### The Test suite: ###

class OptionsCmdline(unittest.TestCase):

	def testUnknownOpts(self):
		"""Options: fail on unknown command line options"""
		try:
			t_opts(argv="test_mp3togo --not-an-option".split())
		except options.Fail:
			return
		raise AssertionError

	def testShortOpt(self):
		"""Options: short options"""
		opts = t_opts(argv="test_mp3togo -y halibut -z 22".split())
		self.assertEqual(opts['test2'], 'halibut')
		self.assertEqual(opts['test3'], 22)
	
	def testLongOpt(self):
		"""Options: long options"""
		opts = t_opts(argv="test_mp3togo --test-2 cod --test-3 57".split())
		self.assertEqual(opts['test2'], 'cod')
		self.assertEqual(opts['test3'], 57)


class OptionsConffile(unittest.TestCase):
	
	def setUp(self):
		clean_config()
	
	def tearDown(self):
		clean_config()

	def testSaveRestore(self):
		"""Options: config file save and restore"""
		try:
			opts = t_opts(argv="test_mp3togo -s -v 0 -y grouper -z 42".split())
		except SystemExit:
			pass
		opts = t_opts()
		self.assertEqual(opts['test2'], 'grouper')
		self.assertEqual(opts['test3'], 42)

	def testConfigFileName(self):
		"""Options: save config to the correct file"""
		try:
			opts = t_opts(argv="test_mp3togo -s -v 0".split())
		except SystemExit:
			pass
		if not os.path.exists(test_conffile):
			raise AssertionError

	def testDontSave(self):
		"""Options: don't save items in _nosave_options"""
		try:
			opts = t_opts(argv="test_mp3togo -s -v 0 --test-no_save 22".split())
		except SystemExit:
			pass
		opts = t_opts()
		self.assertNotEqual(opts['test_nosave'], 22)

	def testIgnoreUnknown(self):
		"""Options: ignore unknown keys in config file"""
		try:
			opts = t_opts(argv="test_mp3togo -s -v 0".split())
		except SystemExit:
			pass
		cf = file(test_conffile, 'a')
		cf.write("lakefish=walleye")
		cf.close()
		opts = t_opts()
		if opts.has_key('lakefish'):
			raise AssertionError

	def testRespectConffile(self):
		"""Options: alternate conffile directive"""
		clean_alt_config()
		try:
			opts = t_opts(argv="test_mp3togo -s -v 0 -y trout".split(), conffile=test_alt_conffile)
		except SystemExit:
			pass
		opts = t_opts(conffile=test_alt_conffile)
		self.assertEqual(opts['test2'], 'trout')
		clean_alt_config()

	def testRespectReadconf(self):
		"""Options: readconf flag"""
		try:
			opts = t_opts(argv="test_mp3togo -s -v 0 -y anchovie".split())
		except SystemExit:
			pass
		opts = t_opts(readconf=False)
		self.assertNotEqual(opts['test2'], 'anchovie')
		

class OptionsValidate(unittest.TestCase):
	
	def testValidateStr(self):
		"""Options: string options save as string"""
		opts = t_opts(argv="test_mp3togo -y 42".split())
		if type(opts['test2']) is not type(''):
			raise AssertionError
		try:
			opts['test2'] = 5
		except ValueError:
			return
		raise AssertionError

	def testValidateInt(self):
		"""Options: integer options save as integer"""
		opts = t_opts(argv="test_mp3togo -z 42".split())
		if type(opts['test3']) is not type(1):
			raise AssertionError
		try:
			opts['test3'] = 'tuna'
		except ValueError:
			return
		raise AssertionError

	def testValidateFloat(self):
		"""Options: float options save as float"""
		opts = t_opts(argv="test_mp3togo --test-4 42.9".split())
		if type(opts['test4']) is not type(1.0):
			raise AssertionError
		try:
			opts['test4'] = 'stingray'
		except ValueError:
			return
		raise AssertionError

	def testValidateBool(self):
		"""Options: boolean options save as bool"""
		opts = t_opts(argv="test_mp3togo --test-4 42.9".split())
		if type(opts['test5']) is not type(True):
			raise AssertionError
		opts['test5'] = False
		if type(opts['test5']) is not type(True):
			raise AssertionError
		try:
			opts['test5'] = 'dogfish'
		except ValueError:
			return
		raise AssertionError


class OptionsHooks(unittest.TestCase):

	def testOptionHook(self):
		"""Options: option hook function"""
		opts = t_opts()
		if not opts['test1'].endswith('ADDED_BY_HOOK'):
			raise AssertionError
		opts['test1'] = 'marlin'
		if not opts['test1'].endswith('ADDED_BY_HOOK'):
			raise AssertionError

	def testPreHook(self):
		"""Options: pre configuration hook executes"""
		opts = t_opts()
		self.assertEqual(opts._pre_hook_ran, True)
	
	def testPostHook(self):
		"""Options: post configuration hook executes"""
		opts = t_opts()
		self.assertEqual(opts._post_hook_ran, True)


class OptionsReturnSettings(unittest.TestCase):
	
	def testValues(self):
		"""Options: option values correct"""
		opts = t_opts()
		self.assertEqual(opts['test1'], 'default-1ADDED_BY_HOOK')
		self.assertEqual(opts['test2'], '')
		self.assertEqual(opts['test3'], 0)
		self.assertEqual(opts['test4'], 0.0)
		self.assertEqual(opts['test5'], False)


class OptionsCheckBin(unittest.TestCase):
	
	def testCheckBin(self):
		"""Options: check for required binaries"""
		opts = t_opts()
		self.assert_(os.path.exists(opts.bin['python']))
		self.assertEqual(opts.bin['This_Binary_Does_Not_Exist'], None)


class OptionsHelp(unittest.TestCase):
	
	def testHelp(self):
		"""Options: produce a help message"""
		opts = t_opts()
		txt = opts.usage()
		self.assertEqual(txt.find("Test 5"), -1)
		self.assertNotEqual(txt.find("Test 4"), -1)
		self.assertNotEqual(txt.find("-y ,   --test-2"), -1)
		

class OptionsLog(unittest.TestCase):
	
	def setUp(self):
		if os.path.exists("./log.out"):
			os.unlink("./log.out")
		self.oldstderr = sys.stderr

	def tearDown(self):
		if os.path.exists("./log.out"):
			os.unlink("./log.out")
		sys.stderr = self.oldstderr

	def testWriteLog(self):
		"""Options: write log messages"""
		fp = file("./log.out", 'w')
		sys.stderr = fp
		opts = t_opts()
		opts.log(1, "one")
		opts.log(2, "two")
		opts['verbosity'] = 3
		opts.log(3, "three")
		opts.log(4, "four")
		fp.close()
		fp = file("./log.out", 'r')
		self.assertEqual(fp.read(), "one\nthree\n")

### End of Tests. ###

def suite():
	suite = unittest.TestSuite()
	suite.addTest(unittest.makeSuite(OptionsCmdline))
	suite.addTest(unittest.makeSuite(OptionsConffile))
	suite.addTest(unittest.makeSuite(OptionsValidate))
	suite.addTest(unittest.makeSuite(OptionsHooks))
	suite.addTest(unittest.makeSuite(OptionsReturnSettings))
	suite.addTest(unittest.makeSuite(OptionsCheckBin))
	suite.addTest(unittest.makeSuite(OptionsHelp))
	suite.addTest(unittest.makeSuite(OptionsLog))
	return suite

if __name__ == "__main__":
	unittest.main()
