#!/usr/bin/python
#
# test_pool.py
# A unit test suite for pool.py
#
# (c) Simeon Veldstra 2006  <reallifesim@gmail.com>
#
# This software is free.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You may redistribute this program under the terms of the
# GNU General Public Licence version 2
# Available in this package or at http://www.fsf.org


"""
Pool must:
Manage an allotment of filesystem space.
Take direction from an Options() instance.
Understand the difference between temp space
and output space. Negative values indicate a
consumption of data.

add_task(task, block):
remove_task(task):
free_space():


"""

import sys, os
import unittest

import test_metadata

# Get the modules from the package we are in, rather
# than any others of the same name installed to the
# system so we can test before installing.
if not sys.path[0] == "../":
	sys.path.insert(0, "../")

import mp3togo.pool as pool
import mp3togo.task as task
import mp3togo.track as track
import mp3togo.conf as conf


args = ['mp3togo',
        '-o', '../../test_data/player',
		'-w', '../../test_data/temp',
		'-v', '0',
		'--no-config-file']

data = test_metadata.metadata()


### Start of Tests ###

class ShortTestPool(unittest.TestCase):
	"""Short tests for pool.Pool()"""

	def setUp(self):
		self.opts = conf.Options(args)
		self.pool = pool.Pool(self.opts)
		self.track = track.Track(data['mp3']['file'], self.opts)

	def testInit(self):
		"""Pool: initialize"""
		td = self.opts['tempdir']
		pd = self.opts['playerdir']
		freetmp = os.statvfs(td).f_bfree * os.statvfs(td).f_frsize
		freeout = os.statvfs(pd).f_bfree * os.statvfs(pd).f_frsize
		
		self.assertEqual(freetmp, self.pool.tmpspace())
		self.assertEqual(freeout, self.pool.outspace())
		if freetmp > os.stat(data['mp3']['file']).st_size * 20:
			self.assertEqual(True, self.pool.add_track(self.track))
			for tsk in self.track._queue:
				self.assert_(tsk in self.pool._active)
		else:
			print "Not enough temporary space to run test."
		if not freeout > os.stat(data['mp3']['file']).st_size * 2:
			print "Not enough output space to run test."

	def testNoOutSpace(self):
		"""Pool: not enough output space"""
		arg2 = args[:]
		arg2.append('--max-size')
		arg2.append('1M')
		opts = conf.Options(arg2)
		trk = track.Track(data['mp3']['file'], opts)
		pl = pool.Pool(opts)
		self.assertEqual(pl.outspace(), 1024 * 1024)
		self.assertEqual(pl.add_track(trk), False)
		self.assertEqual(pl._active, [])

	def testNoTmpSpace(self):
		"""Pool: not enough temporary space"""
		arg2 = args[:]
		arg2.append('--max-temp-size')
		arg2.append('1M')
		opts = conf.Options(arg2)
		trk = track.Track(data['mp3']['file'], opts)
		pl = pool.Pool(opts)
		self.assertEqual(pl.tmpspace(), opts['maxtempsize'])
		self.assertEqual(pl.add_track(trk), False)
		self.assertEqual(pl._active, [])

	def testTaskCalcs(self):
		"""Pool: account for new track"""
		self.pool.add_track(self.track)
		tout = 0
		ttmp = 0 
		for tsk in self.track._queue:
			tout += tsk.outsize
			if tsk.tmpsize > 0:
				ttmp += tsk.tmpsize
		self.assertEqual(self.pool._out[pool.NEED], tout)
		self.assertEqual(self.pool._tmp[pool.NEED], ttmp)

	def testTaskFinished(self):
		"""Pool: account for finished tasks"""
		self.pool.add_track(self.track)
		for tsk in self.track._queue:
			tsk._status = 'done'
		self.track.bytes = 1234567
		self.track._state = track.DONE
		self.pool.rm_track(self.track)
		self.assertEqual(self.pool._out[pool.USED], 1234567)
		self.assertEqual(self.pool._out[pool.NEED], 0)

	def testTaskFailed(self):
		"""Pool: account for failed tasks"""
		self.pool.add_track(self.track)
		for tsk in self.track._queue:
			tsk._status = 'failed'
		self.track._state = track.FAILED
		self.pool.rm_track(self.track)
		self.assertEqual(self.pool._out[pool.USED], 0)
		self.assertEqual(self.pool._out[pool.NEED], 0)

	def testFuture(self):
		"""Pool: look ahead for free tempspace"""
		self.pool.add_track(self.track)
		totalneg = 0
		for tsk in self.pool._active:
			if tsk.tmpsize < 0:
				totalneg += -tsk.tmpsize
		self.assertEqual(totalneg, self.pool.tmp_willgrow())



### End of Tests ###

def suite():
	suite = unittest.TestSuite()
	suite.addTest(unittest.makeSuite(ShortTestPool))
	return suite

if __name__ == "__main__":
	unittest.main()
