#!/usr/bin/python
#
# test_tags.py
# A unit test suite for tags.py
#
# (c) Simeon Veldstra 2006  <reallifesim@gmail.com>
#
# This software is free.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You may redistribute this program under the terms of the
# GNU General Public Licence version 2
# Available in this package or at http://www.fsf.org


"""
Tags must:
Take its input filename from its constructor.
Raise ErrorNoFile if the file doesn't exist.
Raise ErrorUnknownFileType if the file isn't supported.
Read tags in when read() is called.
Write tags to the argument of write(filename).
Write (append to) an index file when writeindex() is called.
Return a formatted string of metadata from format().
Expose the tags in a dictionary interface.
Allow the tags to be modified before writing.

Tag names (keys) must be uppercase, but may be converted
from lowercase input.

init(filename)->waiting->read->ready->modify->write
"""

import sys, os, shutil
import unittest

# Get the modules from the package we are in, rather
# than any others of the same name installed to the
# system so we can test before installing.
if not sys.path[0] == "../":
	sys.path.insert(0, "../")

import mp3togo.tags as tags
import mp3togo.conf as setup


### Set up test data ###

import test_metadata
data = test_metadata.metadata()

### Start of Tests ###

class TagsLoad(unittest.TestCase):
	"""Test initialization"""

	def testInit(self):
		"""Tags: load file"""
		tob = tags.Tags(data['mp3']['file'])
		self.assertEqual(tob._file, data['mp3']['file'])
		self.assertEqual(tob._tags, {})

	def testInitNoFile(self):
		"""Tags: load nonexistant file"""
		try:
			tob = tags.Tags(data['gone']['file'])
		except setup.ErrorNoFile:
			return 
		raise AssertionError
		
	def testInitUnknownFile(self):
		"""Tags: load non music file"""
		try:
			tob = tags.Tags(data['notmp3']['file'])
		except setup.ErrorUnknownFileType:
			return 
		raise AssertionError
		

class TagsRead(unittest.TestCase):
	"""Test metadata read in"""

	def readtype(self, type):
		if not os.path.exists(data[type]['file']):
			print "Test input file %s missing!" % data[type]['file']
			print "%s read test skipped!" % type
			return
		tob = tags.Tags(data[type]['file'])
		tob.read()
		for tag in ('ARTIST', 'TITLE', 'ALBUM'):
			if tob[tag][0] != data[type][tag]:
				print tob[tag][0], data[type][tag], type
				raise AssertionError
			if tob.has_key("TRACKNUMBER"):
				if tob['TRACKNUMBER'][0] != data[type]['TRACKNUMBER']:
					raise AssertionError

	def testReadMp3(self):
		"""Tags: read mp3 tags in"""
		self.readtype('mp3')

	def testReadOgg(self):
		"""Tags: read ogg vorbis tags in"""
		self.readtype('ogg')

	def testReadFlac(self):
		"""Tags: read flac tags in"""
		self.readtype('flac')

	def testReadWav(self):
		"""Tags: wav tags created"""
		self.readtype('wav')


class TagsWrite(unittest.TestCase):
	"""Test metadata write"""
	
	def tearDown(self):
		for type in ('baremp3', 'bareogg'):
			tmpfile = type + '.' + type[-3:]
			if os.path.exists(tmpfile):
				os.unlink(tmpfile)

	def writetype(self, type):
		if (not os.path.exists(data['mp3']['file'])) or \
		  (not os.path.exists(data[type]['file'])):
		  	print "Test files missing!\nTest skipped!"
			return
		tmpfile = type + '.' + type[-3:]
		shutil.copyfile(data[type]['file'], tmpfile)
		tob = tags.Tags(data['mp3']['file'])
		tob.read()
		comment = ["mp3togo unit test"]
		tob['COMMENT'] = comment
		tob.write(tmpfile)

		otags = tags.Tags(data['mp3']['file'])
		ntags = tags.Tags(tmpfile)
		otags.read()
		ntags.read()

		for key in otags.keys():
			if key == "COMMENT":
				self.assertEqual(ntags[key], comment)
			else:
				self.assertEqual(otags[key], ntags[key])
		for tag in ('ARTIST', 'TITLE', 'ALBUM'):
			if ntags[tag][0] != data['mp3'][tag]:
				raise AssertionError
		
	def testWriteMp3(self):
		"""Tags: write tags to mp3 file"""
		self.writetype('baremp3')

	def testWriteOgg(self):
		"""Tags: write tags to ogg vorbis file"""
		self.writetype('bareogg')


class TagsGenreMunge(unittest.TestCase):
	"""Test genre munging for id3"""

	def setUp(self):
		self.have_test_files = False
		if not os.path.exists(data['mp3']['file']):
			return
		self.tmpfile = "GenreMunge.mp3"
		shutil.copyfile(data['mp3']['file'], self.tmpfile)
		if os.path.exists(self.tmpfile):
			self.have_test_files = True
	
	def tearDown(self):
		if os.path.exists(self.tmpfile):
			os.unlink(self.tmpfile)

	def ModifyID3Genre(self, newval, expect):
		if not self.have_test_files:
			print "Test suite data file missing: %s" % data['mp3']['file']
			print "Test skipped"
			return
		tob = tags.Tags(self.tmpfile)
		tob.read()
		tob['GENRE'] = newval
		tob.write(self.tmpfile)
		del tob
		tob = tags.Tags(self.tmpfile)
		tob.read()
		self.assertEqual(tob['GENRE'], expect)
		
	def testModifyID3GenreNumber(self):
		"""Tags: modify ID3 genre name by number"""
		self.ModifyID3Genre(20, ['Alternative'])

	def testModifyID3GenreName(self):
		"""Tags: modify ID3 genre name by name"""
		self.ModifyID3Genre('Alternative', ['Alternative'])


class TagsWriteIndex(unittest.TestCase):
	"""Test index file write"""

	def cleanindex(self):
		if os.path.exists("index.out"):
			os.unlink("index.out")
	
	def setUp(self):
		self.cleanindex()

	def tearDown(self):
		self.cleanindex()

	def testWriteIndex(self):
		"""Tags: write out an index file"""
		tob = tags.Tags(data['mp3']['file'])
		tob.read()
		tob.writeindex(data['mp3']['file'], "index.out")
		self.assert_(os.path.exists("index.out"))
		fp = file("index.out", 'r')
		self.assertEqual(fp.read(), test_metadata.indexfile())
		fp.close()


class TagsPrettyPrint(unittest.TestCase):
	"""Test pretty printed output string"""

	def testFormat(self):
		"""Tags: pretty print tag data"""
		tob = tags.Tags(data['mp3']['file'])
		tob.read()
		d = data['mp3']
		self.assertEqual(tob.format('%a'), d['ARTIST'])
		self.assertEqual(tob.format('%t'), d['TITLE'])
		self.assertEqual(tob.format('%l'), d['ALBUM'])
		self.assertEqual(tob.format('%a -%% %l - %t'), \
		  d['ARTIST'] + ' -% ' + d['ALBUM'] + ' - ' + d['TITLE'])
		try:
			tob.format('%a says this is a %Bad format string!')
		except setup.ErrorBadFormat:
			return
		raise AssertionError


### End of Tests ###

def suite():
	suite = unittest.TestSuite()
	suite.addTest(unittest.makeSuite(TagsLoad))
	suite.addTest(unittest.makeSuite(TagsRead))
	suite.addTest(unittest.makeSuite(TagsWrite))
	suite.addTest(unittest.makeSuite(TagsGenreMunge))
	suite.addTest(unittest.makeSuite(TagsWriteIndex))
	suite.addTest(unittest.makeSuite(TagsPrettyPrint))
	return suite

if __name__ == "__main__":
	unittest.main()
