#!/usr/bin/python
#
# test_task.py
# A unit test suite for task.py
#
# (c) Simeon Veldstra 2006  <reallifesim@gmail.com>
#
# This software is free.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You may redistribute this program under the terms of the
# GNU General Public Licence version 2
# Available in this package or at http://www.fsf.org


"""
Task must:
Encapsulate a lengthy or threaded operation.
Reserve output and scratch space as needed.
Provide:
 start()
 pause() (if possible)
 stop()
 undo()
 status()
 space_required()
 done()
The output of the operation is optionaly run
through a passed in filter object.

"""

import sys
import os
import unittest
import time
import signal

# Get the modules from the package we are in, rather
# than any others of the same name installed to the
# system so we can test before installing.
if not sys.path[0] == "../":
	sys.path.insert(0, "../")

import mp3togo.task as task
import mp3togo.conf as setup


magic = 'Magic-cigaM'

### Start of Tests ###

class CommonMixIn:
	"""Tests shared by SimpleTask and Task"""

	def testInit(self):
		"""CommonTask: initialize"""
		self.assertEqual(self.st._parent, self)
		self.assertEqual(self.st._action, self.action)
		self.assertEqual(self.st._filter, self.filter)
		self.assertEqual(self.st._reverse, self.reverse)
		self.assertEqual(self.st.tmpsize, 1024)
		self.assertEqual(self.st.outsize, 2048)
		self.assertEqual(self.st.name, "test")

	def testRun(self):
		"""CommonTask: run action"""
		self.st.run()
		self.st.wait()
		self.assert_(os.path.exists(magic))
		self.assertEqual(self.st.output(), magic.upper())

	def testRunTwice(self):
		"""CommonTask: run action twice"""
		self.st.run()
		self.st.wait()
		self.assertEqual(self.st.run(), False)

	def testFail(self):
		"""CommonTask: action fails"""
		self.st._action = self.failfunc
		self.assertEqual(self.st.status(), task.READY)
		self.st.run()
		self.st.wait()
		self.assertEqual(self.st.status(), task.FAILED)

	def testUndo(self):
		"""CommonTask: undo action"""
		self.st.run()
		self.st.wait()
		self.assert_(os.path.exists(magic))
		self.assertEqual(self.st.undo(), True)
		self.assert_(not os.path.exists(magic))

	def testUndoUndone(self):
		"""CommonTask: undo undone"""
		try:
			self.st.undo()
		except setup.TaskNotReadyError:
			return
		raise AssertionError

	def testStatus(self):
		"""CommonTask: status output"""
		self.assertEqual(self.st.status(), task.READY)
		self.st.run()
		self.st.wait()
		self.assertEqual(self.st.status(), task.DONE)

	def testSpaceReq(self):
		"""CommonTask: required space"""
		self.assertEqual(self.st.space_req(), (1024, 2048))

	def testOutput(self):
		"""CommonTask: command output"""
		self.assertEqual(self.st.output(), None)
		self.st.run()
		self.st.wait()
		self.assertEqual(self.st.output(), magic.upper())


class TestSimpleTask(unittest.TestCase, CommonMixIn):
	"""Start up a SimpleTask"""

	def tearDown(self):
		if os.path.exists(magic):
			os.unlink(magic)

	def setUp(self):
		self.tearDown()
		def action():
			os.system('touch %s' % magic)
			return magic
		def reverse():
			if os.path.exists(magic):
				os.unlink(magic)
			return True
		def failfunc():
			raise Exception
		self.action = action
		self.filter = lambda x: x.upper()
		self.reverse = reverse
		self.failfunc = failfunc
		self.st = task.SimpleTask(self, action, self.filter, reverse, 1024, 2048, "test")

	def testInvalidAction(self):
		"""SimpleTask: invalid action"""
		try:
			t = task.SimpleTask(self, None)
		except TypeError:
			return
		raise AssertionError
	
	def testInvalidFilter(self):
		"""SimpleTask: invalid filter"""
		try:
			t = task.SimpleTask(self, self.action, filter="octopus")
		except TypeError:
			return
		raise AssertionError
	

class TestTask(unittest.TestCase, CommonMixIn):
	"""Start up a Task"""

	def tearDown(self):
		if os.path.exists(magic):
			os.unlink(magic)

	def setUp(self):
		self.tearDown()
		def reverse():
			if os.path.exists(magic):
				os.unlink(magic)
			return True
		self.action = ['./do_touch_and_echo']
		self.filter = lambda x: x.upper()
		self.reverse = reverse
		self.failfunc = ['ls', 'BadCommandOrFilename']
		self.st = task.Task(self, self.action, self.filter, self.reverse, 1024, 2048, 'test')

	def testInvalidAction(self):
		"""Task: invalid action"""
		try:
			job = task.Task(None, "This is not a list")
			job = task.Task(None, ['MustBeStrings', 5])
		except TypeError:
			return
		raise AssertionError

	def testStop(self):
		"""Task: stop a running task"""
		job = task.Task(None, ['./do_produce_much_output', '1'])
		job.run()
		time.sleep(0.25)
		self.assertEqual(job.stop(), True)
		self.assertEqual(job.status(), task.STOPPED)
		time.sleep(0.25)
		self.assertEqual(job.status(), task.STOPPED)

	def testDoOver(self):
		"""Task: restart a stopped task"""
		reverse = lambda: True
		job = task.Task(None, ['./do_produce_much_output', '1'], None, reverse)
		job.run()
		time.sleep(0.25)
		self.assertEqual(job.stop(), True)
		self.assertEqual(job.status(), task.STOPPED)
		self.assertEqual(job.undo(), True)
		self.assertEqual(job.run(), True)
		job.wait()
		self.assertEqual(job.status(), task.DONE)

	def testKillChild(self):
		"""Task: recover from a killed child"""
		job = task.Task(None, ['./do_produce_much_output', '1'])
		job.run()
		time.sleep(0.125)
		self.assertEqual(job.status(), task.RUNNING)
		os.kill(job._pid, signal.SIGKILL)
		time.sleep(0.125)
		self.assertEqual(job.status(), task.FAILED)


class LongTestTask(unittest.TestCase):
	"""Long duration tests for Task."""

	def tearDown(self):
		if os.path.exists(magic):
			os.unlink(magic)

	def setUp(self):
		self.tearDown()

	def testLongTask(self):
		"""Task(long): run a long program (4 seconds)"""
		seconds = 4
		job = task.Task(None, ['./do_touch_and_echo', str(seconds)])
		it = 0

		job.run()
		while job.status() == task.RUNNING:
			out = job.output()
			if out:
				num = int(out.split(' ')[0])
				if num < it:
					raise AssertionError
				it = num
			time.sleep(0.2)
			
		self.assertEqual(it, seconds)
		self.assertEqual(job.status(), task.DONE)

	def testPause(self):
		"""Task(long): pause a running task (8 seconds)"""
		# This test takes close to the minimum possible time
		# to produce a reliable result
		def filter(inp, cache=[]):
			"""Accumulate input"""
			if not cache:
				cache.append(inp)
			else:
				cache[0] = cache[0] + inp
			return cache[0]

		# Produces a sequence number and the elapsed time as output
		# every 1/100th of a second for arg seconds
		start_time = time.time()
		job = task.Task(None, ['./do_produce_much_output', '2'], filter)
		job.run()
		self.assertEqual(job.status(), task.RUNNING)
		time.sleep(0.25)
		self.assertEqual(job.pause(), True)
		self.assertEqual(job.status(), task.PAUSED)
		time.sleep(2) # buffer holds about 0.5 seconds of data
		self.assertEqual(job.run(), True)
		self.assertEqual(job.status(), task.RUNNING)
		job.wait()
		finish_time = time.time()
		
		# Did it actually pause the command?
		out = job.output()
		out = out.split('\n')[:-1]
		out = map(lambda x: x.split(), out)
		results = dict(out)
		for x in range(0, len(out) - 1):
			# Are the sequence numbers continuous?
			self.assertEqual(int(out[x+1][0]) - int(out[x][0]), 1)
		delta_t = map(lambda x: float(out[x+1][1]) - float(out[x][1]), 
		              range(0, len(out) - 1))
		max_d = max(delta_t)
		max_index = delta_t.index(max_d)
		del delta_t[max_index]
		avg_d = reduce(lambda x, y: x + y, delta_t) / float(len(delta_t))

		# Was there an expected  significant gap in the times?
		self.assert_(1.0 < max_d)
		self.assert_(avg_d < 0.001)


### End of Tests ###

def short_suite():
	suite = unittest.TestSuite()
	suite.addTest(unittest.makeSuite(TestSimpleTask))
	suite.addTest(unittest.makeSuite(TestTask))
	return suite

def suite():
	suite = short_suite()
	suite.addTest(unittest.makeSuite(LongTestTask))
	return suite


if __name__ == "__main__":
	unittest.main()
