#!/usr/bin/python
#
# test_track.py
# A unit test suite for track.py
#
# (c) Simeon Veldstra 2006  <reallifesim@gmail.com>
#
# This software is free.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You may redistribute this program under the terms of the
# GNU General Public Licence version 2
# Available in this package or at http://www.fsf.org


"""
Track must:
Generate filenames for temp and output files.
Estimate temporary and final file sizes.
Generate a series of commands to transform the track.
Take direction from an Options() object.
Be able to roll back the actions on failure.

"""

import sys, os
import unittest
import time

import test_metadata

# Get the modules from the package we are in, rather
# than any others of the same name installed to the
# system so we can test before installing.
if not sys.path[0] == "../":
	sys.path.insert(0, "../")

import mp3togo.track as track
import mp3togo.task as task
import mp3togo.conf as conf
import mp3togo.tags as tags
import mp3togo.helpers


args = ['mp3togo',
        '--output-dir', '../../test_data/player',
		'--work-dir', '../../test_data/temp',
		'--file-tag', '-TEST-SUITE-',
		'--verbose', '0',
		'--no-config-file']

data = test_metadata.metadata()

### Start of Tests ###
def clean_up():
	for ext in ('.mp3', '.ogg', '.wav', '.flac'):
		outname = data['mp3']['file_outname'][:-4] + ext
		if os.path.exists(outname):
			os.unlink(outname)
	dir = os.path.dirname(data['mp3']['file_wavname'])
	for wav in filter(lambda x: x.endswith('.wav'), os.listdir(dir)):
		os.unlink(os.path.join(dir, wav))
	if os.path.exists(os.path.join(data['mp3']['file_outdir'], '2go.index')):
		os.unlink(os.path.join(data['mp3']['file_outdir'], '2go.index'))
	if os.path.exists(data['mp3']['file_outdir']):
		os.rmdir(data['mp3']['file_outdir'])


class ShortTrack(unittest.TestCase):
	"""Short duration tests for track.Track"""

	def setUp(self):
		self.tearDown()
		self.opts = conf.Options(args)
		self.trk = track.Track(data['mp3']['file'], self.opts)

	def tearDown(self):
		clean_up()

	def testInit(self):
		"""Track: generate filenames"""
		self.assertEqual(self.trk._filename, data['mp3']['file'])
		self.assertEqual(self.trk._outname, data['mp3']['file_outname'])
		self.assertEqual(self.trk._outdir, data['mp3']['file_outdir'])
		self.assertEqual(self.trk._wavname, data['mp3']['file_wavname'])

	def testSizes(self):
		"""Track: estimate filesizes"""
		fname = data['mp3']['file']
		mp3size = os.stat(fname).st_size
		exptmp = mp3togo.helpers.est_decoded_size(fname)
		expout = long(exptmp / self.opts['compfactor'])
		tmpsum = sum(map(lambda x: x.tmpsize, self.trk._queue))
		self.assertEqual(tmpsum, 0)

		steps = dict(map(lambda x: (x.name, (x.tmpsize, x.outsize)), 
		                 self.trk._queue))
		self.assert_(-1 < steps['Decoding mp3'][0] - exptmp < 1)
		self.assert_(-1 < steps['Encoding mp3'][1] - expout < 1)
		self.assert_(-1 < steps['Cleaning'][0] -  (-exptmp) < 1)
		del steps['Decoding mp3'], steps['Encoding mp3'], steps['Cleaning']
		self.assertEqual(reduce(lambda o, n: (o[0] + n[0], o[0] + n[1]),
		                        steps.values()), (0, 0))

	def testTaskIter(self):
		"""Track: iterate through tasks"""
		tsk = self.trk.getfirst()
		for i in self.trk._queue:
			self.assertEqual(tsk, i)
			tsk = tsk.next()
		self.assert_(tsk is None)

	def testTaskIterRev(self):
		"""Track: iterate backwards through tasks"""
		tsk = self.trk._queue[-1]
		lst = list(self.trk._queue)
		lst.reverse()
		for i in lst:
			self.assertEqual(tsk, i)
			tsk = tsk.prev()
		self.assert_(tsk is None)

	def testTaskOrder(self):
		"""Track: run tasks out of order"""
		try:
			self.trk._queue[3].run()
		except conf.TaskNotReadyError:
			return
		raise AssertionError

	def testMakeDirs(self):
		"""Track: make output directories"""
		tsk = self.trk.getfirst()
		tsk.run()
		tsk = tsk.next()
		self.assert_(not os.path.exists(data['mp3']['file_outdir']))
		tsk.run()
		self.assert_(os.path.exists(data['mp3']['file_outdir']))
		tsk.undo()
		self.assert_(not os.path.exists(data['mp3']['file_outdir']))

	def testSentinels(self):
		"""Track: maintain state"""
		self.assertEqual(self.trk._state, track.READY)
		self.trk._queue[0].run()
		self.assertEqual(self.trk._state, track.RUNNING)
		for tsk in self.trk._queue[1:-1]:
			tsk._status = task.DONE
		self.trk._queue[-1].run()
		self.assertEqual(self.trk._state, track.DONE)
		self.trk._queue[-1].undo()
		self.assertEqual(self.trk._state, track.RUNNING)
		self.trk._queue[0].undo()
		self.assertEqual(self.trk._state, track.FAILED)
	
	def testIndex(self):
		"""Track: write and remove an existing index file"""
		iargs = args[:]
		iargs.append("--index")
		opts = conf.Options(iargs)
		trk = track.Track(data['mp3']['file'], opts)
		ifname = os.path.join(trk._outdir, '2go.index')
		names = map(lambda x: x.name, trk._queue)
		md = names.index('Creating dirs')
		wi = names.index('Writing index')
		trk._queue[md - 1]._status = task.DONE
		trk._queue[md].run()

		inttags = tags.Tags(data['mp3']['file'])
		inttags.read()
		inttags['TITLE'] = "Fake Song"
		inttags.writeindex("/this/is/a/fake/file.mp3", ifname)
		inttags.writeindex("/this/is/another/fake/file.mp3", ifname)
		fp = file(ifname, 'r')
		beflines = fp.readlines()
		fp.close()
		os.unlink(ifname)

		inttags.writeindex("/this/is/a/fake/file.mp3", ifname)
		trk._queue[wi - 1]._status = task.DONE
		trk._queue[wi].run()
		inttags.writeindex("/this/is/another/fake/file.mp3", ifname)
		fp = file(ifname, 'r')
		lines = fp.readlines()
		fp.close()

		self.assert_(len(lines) > len(beflines))

		trk._queue[wi].undo()
		fp = file(ifname, 'r')
		aftlines = fp.readlines()
		fp.close()
		os.unlink(ifname)

		self.assertEqual(beflines, aftlines)
	
	def testNewIndex(self):
		"""Track: write and remove a new index file"""
		iargs = args[:]
		iargs.append("--index")
		opts = conf.Options(iargs)
		trk = track.Track(data['mp3']['file'], opts)
		ifname = os.path.join(trk._outdir, '2go.index')
		names = map(lambda x: x.name, trk._queue)
		md = names.index('Creating dirs')
		wi = names.index('Writing index')
		trk._queue[md - 1]._status = task.DONE
		trk._queue[md].run()
		self.assert_(not os.path.exists(ifname))

		trk._queue[wi - 1]._status = task.DONE
		trk._queue[wi].run()
		self.assert_(os.path.exists(ifname))

		trk._queue[wi].undo()
		self.assert_(not os.path.exists(ifname))
		



class LongTrack(unittest.TestCase):
	"""Long duration tests for track.Track"""

	def testRunAndUndo(self):
		"""Track(long): run Track then undo (several minutes)"""
		return # Too expensive, test later more comprehensively
		long_args = args[:]
		long_args.append('--index')
		opts = conf.Options(long_args)
		trk = track.Track(data['mp3']['file'], opts)

class ParseTrack(unittest.TestCase):
	"""Tests for helper functions to parse progress information(long)."""

	def setUp(self):
		clean_up()
		self.opts = conf.Options(args)

	def tearDown(self):
		clean_up()
		
	def testOggDecoder(self):
		"""Track(long): parse program output:  ogg123"""
		self.run_decoder('ogg')
		self.tsk.stop()

	def testFlacDecoder(self):
		"""Track(long): parse program output:  flac --decode"""
		self.run_decoder('flac')
		self.tsk.stop()

	def testMpgEncs(self):
		"""Track(long): parse mpg321, lame, oggenc (30 seconds)"""
		self.run_decoder('mp3')
		self.tsk.wait()

		# lame:
		self.find_task('Encoding')
		self.tsk.run()
		self.check_growth()
		self.tsk.stop()

		# oggenc
		args2 = args[:]
		args2.append('--encoder')
		args2.append('oggenc')
		opts2 = conf.Options(args2)
		otrack = track.Track(data['mp3']['file'], opts2)
		self.tsk = otrack.getfirst()

		self.find_task('Encoding')
		self.tsk.run()
		self.check_growth()
		self.tsk.stop()
		

	def find_task(self, name):
		while not self.tsk.name.startswith(name):
			self.tsk.run()
			self.tsk.wait()
			self.tsk = self.tsk.next()

	def run_decoder(self, name):
		trk = track.Track(data[name]['file'], self.opts)
		self.tsk = trk.getfirst()
		self.find_task('Decoding')
		self.tsk.run()
		self.check_growth()

	def check_growth(self):
		op = self.tsk.output()
		for i in range(16):
			time.sleep(0.125)
			np = self.tsk.output()
			if np and op:
				self.assert_(np >= op)
			if np:
				op = np
		self.assert_(op > 0)

	
### End of Tests ###

def short_suite():
	suite = unittest.TestSuite()
	suite.addTest(unittest.makeSuite(ShortTrack))
	return suite

def suite():
	suite = short_suite()
	# Long duration tests go here
	#suite.addTest(unittest.makeSuite(LongTrack))
	suite.addTest(unittest.makeSuite(ParseTrack))
	return suite


if __name__ == "__main__":
	unittest.main()

